package edu.lawrence.androidcurrency

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material3.Button
import androidx.compose.material3.CenterAlignedTopAppBar
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import edu.lawrence.androidcurrency.ui.theme.AndroidCurrencyTheme

val rates = mapOf("EUR" to 0.9829,"GBP" to 0.8397,"JPY" to 135.72,"CAD" to 1.3049)

@OptIn(ExperimentalMaterial3Api::class)
class MainActivity : ComponentActivity() {
    @OptIn(ExperimentalMaterial3Api::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        setContent {
            AndroidCurrencyTheme {
                CurrencyApp(modifier = Modifier.fillMaxSize())
            }
        }
    }
}

@Composable
fun CurrencyApp(modifier : Modifier) {
    val navController = rememberNavController()
    NavHost(navController,startDestination="calculator/EUR") {
        composable(route="calculator/{code}",
            arguments = listOf(navArgument("code") {
                type = NavType.StringType
            })) { navStackEntry ->
            val code = navStackEntry.arguments?.getString("code")
            CalculatorScreen(code?:"EUR",toOptions = { navController.navigate("currencies")}, modifier = modifier)
        }
        composable(route="currencies") {
            CurrenciesScreen({code : String -> navController.navigate("calculator/$code")}, modifier = modifier)
        }
    }
}

@Composable
fun CurrenciesScreen(onClick : (String)->Unit, modifier : Modifier) {
    Column(verticalArrangement = Arrangement.Center,modifier = modifier.padding(20.dp)) {
        Row(horizontalArrangement = Arrangement.Center, modifier = Modifier.padding(20.dp)) {
            Text("Select a currency")
        }
        HorizontalDivider()
        Column(verticalArrangement = Arrangement.Center, modifier = Modifier.padding(20.dp)) {
            rates.keys.forEach { code ->
                CurrencyItem(code = code, onClick = onClick)
            }
        }
    }
}

@Composable
fun CurrencyItem(code : String, onClick : (String)->Unit) {
    Text(text = code, modifier=Modifier.clickable { onClick(code) })
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CalculatorScreen(code : String, toOptions: () -> Unit,modifier: Modifier) {
    Scaffold(topBar = {
        TopAppBar(
            title = {
                Text(
                    "Travel Money",
                )
            },
            actions = {
                IconButton(onClick = toOptions) {
                    Icon(
                        imageVector = Icons.Default.Settings,
                        contentDescription = "Settings"
                    )
                }
            },
        )
    },
        modifier = Modifier.fillMaxSize())
    { innerPadding ->
        Calculator(code,modifier.padding(innerPadding))
    }
}

@Composable
fun Calculator(code : String,modifier : Modifier) {
    val focusManager = LocalFocusManager.current
    val amount = remember { mutableStateOf("100") }
    val converted = remember { mutableStateOf("%.2f".format(rates[code]!!*100))}
    fun convert() : Unit {
        val dollarAmount = amount.value.toDoubleOrNull()
        if(dollarAmount == null) {
            converted.value = "unknown"
        } else {
            val exchangeAmount = rates[code]!!*dollarAmount!!
            converted.value = "%.2f".format(exchangeAmount)
        }
    }

    Column(horizontalAlignment = Alignment.CenterHorizontally, modifier = modifier.fillMaxSize()) {
        Row(horizontalArrangement = Arrangement.Start, verticalAlignment = Alignment.Bottom) {
            TextField(value = amount.value,
                onValueChange = {newValue : String -> amount.value = newValue},
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() }))
            Spacer(modifier=Modifier.width(10.dp))
            Text("USD")
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Button(onClick = {convert()}) {
                Text("Convert")
            }
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Text(converted.value)
            Text(code)
        }

    }
}

@Preview(showBackground = true)
@Composable
fun CurrencyPreview() {
    AndroidCurrencyTheme {
        Calculator("EUR",modifier = Modifier.fillMaxSize())
    }
}